/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "HTMLFormElement.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomHTMLElement.h"
#include "gdom/GdomHTMLFormElement.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomHTMLElement.h"
#include "gdom/PrivateGdomHTMLFormElement.h"
#include "gdom/PrivateIntGdomHTMLFormElement.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLFormElement::GDOMHTMLFormElement(PassRefPtr<WebCore::HTMLFormElement> impl)
    : GDOMHTMLElement(impl)
{
}

void
gdom_html_form_element_submit (GdomHTMLFormElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLFormElement * item = (gdom_html_form_element_get_private(thiz));
    g_return_if_fail (item);
    item->submit();

}

void
gdom_html_form_element_reset (GdomHTMLFormElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::HTMLFormElement * item = (gdom_html_form_element_get_private(thiz));
    g_return_if_fail (item);
    item->reset();

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLFormElement.h"
#include <gdom/GdomHTMLFormElement.h>
#include <gdom/PrivateGdomHTMLFormElement.h>
#include <gdom/PrivateIntGdomHTMLFormElement.h>
#include <gdom/GdomHTMLFormElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLFormElement, gdom_html_form_element, GDOM_TYPE_HTML_ELEMENT);

/*
struct _GdomHTMLFormElementPrivate {
    class WebCore::HTMLFormElement *item;
};

#define GDOM_HTML_FORM_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_FORM_ELEMENT, GdomHTMLFormElementPrivate))
*/
G_END_DECLS
WebCore::HTMLFormElement *
gdom_html_form_element_get_private(GdomHTMLFormElement *request)
{
    GDOMHTMLFormElement *res;
    res = static_cast<GDOMHTMLFormElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLFormElement*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_ELEMENTS,
    PROP_LENGTH,
    PROP_NAME,
    PROP_ACCEPT_CHARSET,
    PROP_ACTION,
    PROP_ENCODING,
    PROP_ENCTYPE,
    PROP_METHOD,
    PROP_TARGET,
};


static void gdom_html_form_element_finalize(GObject* object)
{
    /*GdomHTMLFormElement* request = GDOM_HTML_FORM_ELEMENT(object);*/
    /*GdomHTMLFormElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_HTML_ELEMENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_form_element_parent_class)->finalize(object);
}

static void gdom_html_form_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLFormElement *self = GDOM_HTML_FORM_ELEMENT(object);
    WebCore::HTMLFormElement * gself = (gdom_html_form_element_get_private(self));
    switch (prop_id) {
     case PROP_NAME:
    {
         gself->setName(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ACCEPT_CHARSET:
    {
         gself->setAcceptCharset(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ACTION:
    {
         gself->setAction(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ENCODING:
    {
         gself->setEncoding(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_ENCTYPE:
    {
         gself->setEnctype(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_METHOD:
    {
         gself->setMethod(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_TARGET:
    {
         gself->setTarget(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_form_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLFormElement *self = GDOM_HTML_FORM_ELEMENT(object);
    WebCore::HTMLFormElement * gself = (gdom_html_form_element_get_private(self));
    switch (prop_id) {
    case PROP_ELEMENTS:
    {
         PassRefPtr<WebCore::HTMLCollection> ptr = gself->elements();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_LENGTH:
    {
         g_value_set_long (value, gself->length());
         break;
    }
    case PROP_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->name()));
         break;
    }
    case PROP_ACCEPT_CHARSET:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->acceptCharset()));
         break;
    }
    case PROP_ACTION:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->action()));
         break;
    }
    case PROP_ENCODING:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->encoding()));
         break;
    }
    case PROP_ENCTYPE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->enctype()));
         break;
    }
    case PROP_METHOD:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->method()));
         break;
    }
    case PROP_TARGET:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->target()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_form_element_class_init(GdomHTMLFormElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_form_element_finalize;
    g_klass->set_property = gdom_html_form_element__set_property_fn;
    g_klass->get_property = gdom_html_form_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_ELEMENTS,
            g_param_spec_object ("elements", /* name */
                "html_form_element_elements", /* short description */
                "read-only  GdomHTMLCollection * HTMLFormElement.elements", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_COLLECTION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_long ("length", /* name */
                "html_form_element_length", /* short description */
                "read-only  glong HTMLFormElement.length", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NAME,
            g_param_spec_string ("name", /* name */
                "html_form_element_name", /* short description */
                "read-write  gchar * HTMLFormElement.name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ACCEPT_CHARSET,
            g_param_spec_string ("accept_charset", /* name */
                "html_form_element_accept_charset", /* short description */
                "read-write  gchar * HTMLFormElement.accept_charset", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ACTION,
            g_param_spec_string ("action", /* name */
                "html_form_element_action", /* short description */
                "read-write  gchar * HTMLFormElement.action", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ENCODING,
            g_param_spec_string ("encoding", /* name */
                "html_form_element_encoding", /* short description */
                "read-write  gchar * HTMLFormElement.encoding", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_ENCTYPE,
            g_param_spec_string ("enctype", /* name */
                "html_form_element_enctype", /* short description */
                "read-write  gchar * HTMLFormElement.enctype", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_METHOD,
            g_param_spec_string ("method", /* name */
                "html_form_element_method", /* short description */
                "read-write  gchar * HTMLFormElement.method", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TARGET,
            g_param_spec_string ("target", /* name */
                "html_form_element_target", /* short description */
                "read-write  gchar * HTMLFormElement.target", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLFormElementPrivate));*/
}

static void gdom_html_form_element_init(GdomHTMLFormElement* request)
{
    /*GdomHTMLFormElementPrivate* priv = GDOM_HTML_FORM_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

